#include "interactive/interactive.hpp"
#include <ncurses.h>
#include <string>
#include <vector>
#include <chrono>
#include <thread>
#include <algorithm>

namespace interactive {

int fullscreen_window::ncurses_streambuf::overflow(int c) {
    if (c != EOF) {
        buffer += static_cast<char>(c);
        if (c == '\n') {
            wprintw(win, "%s", buffer.c_str());
            wrefresh(win);
            buffer.clear();
        }
    }
    return c;
}

fullscreen_window::fullscreen_window(std::string title) {
    // Initialize ncurses
    initscr();
    raw();  // Use raw mode for better control
    noecho();
    keypad(stdscr, TRUE);  // Enable keypad
    curs_set(0);  // Hide cursor
    refresh();

    // Create display window (takes up all but bottom 4 lines)
    int max_y, max_x;
    getmaxyx(stdscr, max_y, max_x);
    display_win = newwin(max_y - 4, max_x, 0, 0);
    scrollok(display_win, TRUE);
    keypad(display_win, TRUE);  // Enable keypad for display window
    wrefresh(display_win);

    // Create input window (bottom 4 lines)
    input_win = newwin(4, max_x, max_y - 4, 0);
    box(input_win, 0, 0);
    keypad(input_win, TRUE);  // Enable keypad for input window
    wrefresh(input_win);

    // Set up output redirection
    cout_buf = new ncurses_streambuf(display_win);
    cerr_buf = new ncurses_streambuf(display_win);
    
    // Save original streambufs
    original_cout = std::cout.rdbuf();
    original_cerr = std::cerr.rdbuf();
    
    // Redirect cout and cerr to our custom streambuf
    std::cout.rdbuf(cout_buf);
    std::cerr.rdbuf(cerr_buf);
}

void fullscreen_window::close() {
    if (is_closed) return;
    is_closed = true;

    // Restore original streambufs
    std::cout.rdbuf(original_cout);
    std::cerr.rdbuf(original_cerr);
    
    // Clean up streambufs
    delete cout_buf;
    delete cerr_buf;
    
    // Clean up ncurses windows
    delwin(display_win);
    delwin(input_win);
    
    // End ncurses
    endwin();
}

fullscreen_window::~fullscreen_window() {
    close();
}

void fullscreen_window::set_input_text_display(std::string text) {
    werase(input_win);
    box(input_win, 0, 0);
    
    // Center the text
    int max_y, max_x;
    getmaxyx(input_win, max_y, max_x);
    int start_x = (max_x - text.length()) / 2;
    mvwprintw(input_win, max_y/2, start_x, "%s", text.c_str());
    
    wrefresh(input_win);
}

std::string fullscreen_window::set_input_text_entry(std::string prompt) {
    werase(input_win);
    box(input_win, 0, 0);
    
    // Display prompt
    mvwprintw(input_win, 1, 2, "%s", prompt.c_str());
    wrefresh(input_win);
    
    // Set up input
    echo();
    curs_set(1);
    std::string input;
    int ch;
    int x = prompt.length() + 3;
    
    while ((ch = wgetch(input_win)) != '\n') {
        if (ch == 27) { // ESC key
            input = "";
            break;
        }
        if (ch == KEY_BACKSPACE || ch == 127) {
            if (!input.empty()) {
                input.pop_back();
                x--;
                mvwdelch(input_win, 1, x);
            }
        } else if (isprint(ch)) {
            input += ch;
            mvwaddch(input_win, 1, x++, ch);
        }
    }
    
    noecho();
    curs_set(0);
    return input;
}

std::string fullscreen_window::set_input_multiple_choice(std::string prompt, std::vector<std::string> choices) {
    werase(input_win);
    box(input_win, 0, 0);
    
    // Display prompt
    mvwprintw(input_win, 1, 2, "%s", prompt.c_str());
    
    int selected = 0;
    std::string filter = "";
    auto last_key_time = std::chrono::steady_clock::now();
    int scroll_offset = 0;
    std::vector<std::string> last_filtered_choices;
    int last_selected = -1;
    int last_scroll_offset = -1;
    
    while (true) {
        // Filter choices based on input
        std::vector<std::string> filtered_choices;
        for (const auto& choice : choices) {
            if (filter.empty() || choice.find(filter) == 0) {
                filtered_choices.push_back(choice);
            }
        }
        
        if (filtered_choices.empty()) {
            filtered_choices = choices;
            filter = "";
        }
        
        // Get window dimensions
        int max_y, max_x;
        getmaxyx(input_win, max_y, max_x);
        int available_width = max_x - 4; // -4 for borders
        
        // Calculate scroll offset to keep selected item visible
        if (selected >= 0 && selected < filtered_choices.size()) {
            int item_start = 0;
            for (int i = 0; i < selected; i++) {
                item_start += filtered_choices[i].length() + 3;
            }
            int item_end = item_start + filtered_choices[selected].length();
            
            if (item_start < scroll_offset) {
                scroll_offset = item_start;
            } else if (item_end > scroll_offset + available_width) {
                scroll_offset = item_end - available_width;
            }
        }
        
        // Only redraw if something changed
        if (filtered_choices != last_filtered_choices || 
            selected != last_selected || 
            scroll_offset != last_scroll_offset) {
            
            // Clear the input window except for the prompt
            werase(input_win);
            box(input_win, 0, 0);
            mvwprintw(input_win, 1, 2, "%s", prompt.c_str());
            
            // Display filtered choices horizontally
            int x = 2 - scroll_offset;
            for (size_t i = 0; i < filtered_choices.size(); i++) {
                if (i == selected) {
                    wattron(input_win, A_REVERSE);
                }
                
                // Only draw if visible
                if (x + filtered_choices[i].length() > 0 && x < max_x - 2) {
                    mvwprintw(input_win, 2, x, "%s", filtered_choices[i].c_str());
                }
                
                if (i == selected) {
                    wattroff(input_win, A_REVERSE);
                }
                
                x += filtered_choices[i].length() + 3; // +3 for " | " separator
                
                // Draw separator if not last item
                if (i < filtered_choices.size() - 1) {
                    if (x - 3 + 3 > 0 && x - 3 < max_x - 2) {
                        mvwprintw(input_win, 2, x - 3, " | ");
                    }
                }
            }
            
            wrefresh(input_win);
            
            // Update last state
            last_filtered_choices = filtered_choices;
            last_selected = selected;
            last_scroll_offset = scroll_offset;
        }
        
        int ch = wgetch(input_win);
        
        // Handle key input
        switch (ch) {
            case '\n':
                if (!filtered_choices.empty()) {
                    return filtered_choices[selected];
                }
                break;
                
            case 27: // ESC key
                if (!filter.empty()) {
                    // Clear the filter and reset selection
                    filter = "";
                    selected = 0;
                    scroll_offset = 0;
                } else {
                    // Only return if filter is already empty
                    return "";
                }
                break;
                
            case KEY_LEFT:
            case KEY_UP:
                if (selected > 0) {
                    selected--;
                } else {
                    selected = filtered_choices.size() - 1; // Wrap to end
                }
                break;
                
            case KEY_RIGHT:
            case KEY_DOWN:
                if (selected < filtered_choices.size() - 1) {
                    selected++;
                } else {
                    selected = 0; // Wrap to beginning
                }
                break;
                
            default:
                if (isalnum(ch)) {
                    auto now = std::chrono::steady_clock::now();
                    auto elapsed = std::chrono::duration_cast<std::chrono::milliseconds>(now - last_key_time).count();
                    
                    if (elapsed > 500) { // Reset filter if too much time has passed
                        filter = "";
                    }
                    
                    filter += ch;
                    selected = 0;
                    last_key_time = now;
                }
                break;
        }
    }
}

bool fullscreen_window::set_input_yes_no(std::string prompt) {
    werase(input_win);
    box(input_win, 0, 0);
    
    // Display prompt
    mvwprintw(input_win, 1, 2, "%s", prompt.c_str());
    
    int selected = 0; // 0 for Yes, 1 for No
    const char* options[] = {"Yes", "No"};
    
    while (true) {
        // Display options
        for (int i = 0; i < 2; i++) {
            if (i == selected) {
                wattron(input_win, A_REVERSE);
            }
            mvwprintw(input_win, 2, 2 + i * 10, "%s", options[i]);
            if (i == selected) {
                wattroff(input_win, A_REVERSE);
            }
        }
        
        wrefresh(input_win);
        
        int ch = wgetch(input_win);
        
        if (ch == '\n') {
            return selected == 0;
        } else if (ch == 'y' || ch == 'Y') {
            return true;
        } else if (ch == 'n' || ch == 'N') {
            return false;
        } else if (ch == KEY_LEFT) {
            selected = 0;
        } else if (ch == KEY_RIGHT) {
            selected = 1;
        }
    }
}

char fullscreen_window::set_input_yes_no_cancel(std::string prompt) {
    werase(input_win);
    box(input_win, 0, 0);
    
    // Display prompt
    mvwprintw(input_win, 1, 2, "%s", prompt.c_str());
    
    int selected = 0; // 0 for Yes, 1 for No, 2 for Cancel
    const char* options[] = {"Yes", "No", "Cancel"};
    
    while (true) {
        // Display options
        for (int i = 0; i < 3; i++) {
            if (i == selected) {
                wattron(input_win, A_REVERSE);
            }
            mvwprintw(input_win, 2, 2 + i * 10, "%s", options[i]);
            if (i == selected) {
                wattroff(input_win, A_REVERSE);
            }
        }
        
        wrefresh(input_win);
        
        int ch = wgetch(input_win);
        
        if (ch == '\n') {
            return selected == 0 ? 'y' : (selected == 1 ? 'n' : 'c');
        } else if (ch == 'y' || ch == 'Y') {
            return 'y';
        } else if (ch == 'n' || ch == 'N') {
            return 'n';
        } else if (ch == 'c' || ch == 'C') {
            return 'c';
        } else if (ch == KEY_LEFT) {
            selected = (selected - 1 + 3) % 3;
        } else if (ch == KEY_RIGHT) {
            selected = (selected + 1) % 3;
        }
    }
}

} // namespace interactive