#ifndef INTERACTIVE_HPP
#define INTERACTIVE_HPP

#include <string>
#include <vector>
#include <ncurses.h>
#include <streambuf>
#include <iostream>

namespace interactive {

class fullscreen_window {
    private:
        WINDOW* display_win;
        WINDOW* input_win;
        
        // Original streambufs to restore later
        std::streambuf* original_cout;
        std::streambuf* original_cerr;
        
        // Custom streambuf for redirecting output
        class ncurses_streambuf : public std::streambuf {
            private:
                WINDOW* win;
                std::string buffer;
                
            protected:
                virtual int overflow(int c) override;
                
            public:
                ncurses_streambuf(WINDOW* w) : win(w) {}
        };
        
        ncurses_streambuf* cout_buf;
        ncurses_streambuf* cerr_buf;
        bool is_closed = false;

    public:
    // uses ncurses to create a fullscreen text area, with two windows:
    // one (display) that shows everything sent to stdout or stderr, displaying any text output from the program
    // and the other (input) for interactive input
    // the input window is always at the bottom of the screen, and the display window takes up the rest of the screen
    // the input window is 4 rows high, and the display window takes up the rest of the screen
    // the two windows take up the entire screen, and the program can handle resizing
    // pressing the ` key at any time will bring up a confirmation dialog to quit the program
    fullscreen_window(std::string title);
    
    // Destructor to restore original streambufs
    ~fullscreen_window();
    
    // just display centered text in the input window. returns immediately.
    void set_input_text_display(std::string text); 


    // displays the prompt and a text entry box.
    // only returns after the user has entered text and pressed enter. pressing escape returns an empty string.
    std::string set_input_text_entry(std::string prompt); 
    

    // displays the prompt and then a list of choices.
    // returns when user selects a choice using arrow keys and presses enter. pressing escape returns an empty string.
    // letter keys filter by choices to those that start with that letter, and if multiple letters are typed with a delay it will filter by that string of letters.
    std::string set_input_multiple_choice(std::string prompt, std::vector<std::string> choices); 

    // displays a yes/no confirmation prompt, with arrow keys to select yes or no. returns true if yes, false if no.
    // allows y as shortcut for yes and n as shortcut for no.
    bool set_input_yes_no(std::string prompt);

    // displays a yes/no/cancel confirmation prompt, with arrow keys to select yes, no, or cancel.
    // allows y as shortcut for yes, n as shortcut for no, and c as shortcut for cancel.
    // returns y for yes, n for no, and c for cancel.
    char set_input_yes_no_cancel(std::string prompt);

    // closes the fullscreen window and restores the original streambufs
    void close();
};

} // namespace interactive

#endif
